/******************************************************************************
*                                                                             *
*    Colors.cpp                             Copyright(c) 2007-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include "LafPlayer.h"
#include "Colors.h"
#include "Util.h"
#include "resource.h"


static const struct {
	COLORREF crColor;
	LPCTSTR pszName;
} ColorList[] = {
	{RGB(0,0,0),				TEXT("Black")},
	{RGB(128,0,0),				TEXT("Brown")},
	{RGB(0,128,0),				TEXT("Green")},
	{RGB(128,128,0),			TEXT("Olive")},
	{RGB(0,0,128),				TEXT("DarkBlue")},
	{RGB(128,0,128),			TEXT("DarkMagenta")},
	{RGB(0,128,128),			TEXT("DarkCyan")},
	{RGB(192,192,192),			TEXT("Gray")},
	{RGB(128,128,128),			TEXT("Silver")},
	{RGB(255,0,0),				TEXT("Red")},
	{RGB(0,255,0),				TEXT("Lime")},
	{RGB(255,255,0),			TEXT("Yellow")},
	{RGB(0,0,255),				TEXT("Blue")},
	{RGB(255,0,255),			TEXT("Magenta")},
	{RGB(0,255,255),			TEXT("Cyan")},
	{RGB(255,255,255),			TEXT("White")},
	{COLOR_WINDOW,				TEXT("Window")},
	{COLOR_WINDOWTEXT,			TEXT("WindowText")},
	{COLOR_HIGHLIGHT,			TEXT("Highlight")},
	{COLOR_HIGHLIGHTTEXT,		TEXT("HighlightText")},
	{COLOR_APPWORKSPACE,		TEXT("AppWorkSpace")},
	{COLOR_DESKTOP,				TEXT("Desktop")},
	{COLOR_3DFACE,				TEXT("3DFace")},
	{COLOR_3DLIGHT,				TEXT("3DLight")},
	{COLOR_3DHIGHLIGHT,			TEXT("3DHighlight")},
	{COLOR_3DSHADOW,			TEXT("3DShadow")},
	{COLOR_3DDKSHADOW,			TEXT("3DDarkShadow")},
	{COLOR_BTNTEXT,				TEXT("ButtonText")},
	{COLOR_MENU,				TEXT("Menu")},
	{COLOR_MENUTEXT,			TEXT("MenuText")},
	{COLOR_ACTIVECAPTION,		TEXT("ActiveCaption")},
	{COLOR_CAPTIONTEXT,			TEXT("ActiveCaptionText")},
	{COLOR_INACTIVECAPTION,		TEXT("InactiveCaption")},
	{COLOR_INACTIVECAPTIONTEXT,	TEXT("InactiveCaptionText")},
	{COLOR_INFOBK,				TEXT("InfoBack")},
	{COLOR_INFOTEXT,			TEXT("InfoText")},
};




COLORREF ColorInfo_GetColor(ColorInfo Color)
{
	ColorType Type=ColorInfo_GetType(Color);
	COLORREF cr;

	if (Type==COLOR_TYPE_CUSTOM)
		cr=ColorInfo_GetCustomColor(Color);
	else if (Type<COLOR_TYPE_SYSCOLORFIRST)
		cr=ColorList[Type-1].crColor;
	else
		cr=GetSysColor(ColorList[Type-1].crColor);
	return cr;
}


LPTSTR ColorInfo_GetText(ColorInfo Color,LPTSTR pszText)
{
	ColorType Type=ColorInfo_GetType(Color);

	if (Type==COLOR_TYPE_CUSTOM)
		wsprintf(pszText,TEXT("#%02X%02X%02X"),
			ColorInfo_GetCustomRed(Color),ColorInfo_GetCustomGreen(Color),
											ColorInfo_GetCustomBlue(Color));
	else
		lstrcpy(pszText,ColorList[Type-1].pszName);
	return pszText;
}


BOOL ColorInfo_Parse(ColorInfo *pColor,LPCTSTR pszText)
{
	if (pszText[0]=='#') {
		unsigned int uValue;

		if (lstrlen(pszText+1)!=6)
			return FALSE;
		uValue=StrToUInt(pszText+1,16);
		*pColor=PackColorInfo(COLOR_TYPE_CUSTOM,
							(uValue>>16)&0xFF,(uValue>>8)&0xFF,uValue&0xFF);
	} else {
		int i;

		for (i=0;i<numberof(ColorList);i++)
			if (lstrcmpi(ColorList[i].pszName,pszText)==0)
				break;
		if (i==numberof(ColorList))
			return FALSE;
		ColorInfo_SetType(pColor,i+1);
	}
	return TRUE;
}


BOOL ColorInfo_EqualColor(ColorInfo Color1,ColorInfo Color2)
{
	ColorType Type1,Type2;

	Type1=ColorInfo_GetType(Color1);
	Type2=ColorInfo_GetType(Color2);
	return Type1==Type2 && (Type1!=COLOR_TYPE_CUSTOM || Color1==Color2);
}


void ColorsComboBox_Init(HWND hDlg,int nID,ColorInfo Color)
{
	HWND hwnd=GetDlgItem(hDlg,nID);
	HINSTANCE hinst=GetResourceInstance();
	TCHAR szText[64];
	int i;

	LoadString(hinst,IDS_COLOR_FIRST,szText,numberof(szText));
	(void)ComboBox_SetItemData(hwnd,ComboBox_AddString(hwnd,szText),
											ColorInfo_GetCustomColor(Color));
	for (i=0;i<numberof(ColorList);i++) {
		LoadString(hinst,IDS_COLOR_FIRST+i+1,szText,numberof(szText));
		(void)ComboBox_SetItemData(hwnd,ComboBox_AddString(hwnd,szText),i+1);
	}
	(void)ComboBox_SetCurSel(hwnd,ColorInfo_GetType(Color));
}


ColorInfo ColorsComboBox_GetColor(HWND hDlg,int nID)
{
	HWND hwnd=GetDlgItem(hDlg,nID);
	int nSel=ComboBox_GetCurSel(hwnd);
	COLORREF cr;

	if (nSel==CB_ERR)
		nSel=0;
	cr=(COLORREF)ComboBox_GetItemData(hwnd,0);
	return PackColorInfo(nSel,GetRValue(cr),GetGValue(cr),GetBValue(cr));
}


void ColorsComboBox_SetColor(HWND hDlg,int nID,ColorInfo Color)
{
	HWND hwnd=GetDlgItem(hDlg,nID);

	(void)ComboBox_SetItemData(hwnd,0,ColorInfo_GetCustomColor(Color));
	(void)ComboBox_SetCurSel(hwnd,ColorInfo_GetType(Color));
	InvalidateRect(hwnd,NULL,TRUE);
}


BOOL ColorsComboBox_OnMeasureItem(HWND /*hDlg*/,WPARAM /*wParam*/,LPARAM lParam)
{
	/*
	HWND hwnd=GetDlgItem(hDlg,wParam);
	HDC hdc;
	TEXTMETRIC tm;

	hdc=GetDC(hDlg);
	GetTextMetrics(hdc,&tm);
	ReleaseDC(hwnd,hdc);
	((LPMEASUREITEMSTRUCT)lParam)->itemHeight=tm.tmHeight+2;
	*/
	((LPMEASUREITEMSTRUCT)lParam)->itemHeight=
											7*HIWORD(GetDialogBaseUnits())/8;
	return TRUE;
}


BOOL ColorsComboBox_OnDrawItem(HWND hDlg,WPARAM wParam,LPARAM lParam)
{
	HWND hwnd=GetDlgItem(hDlg,wParam);
	LPDRAWITEMSTRUCT pdis=(LPDRAWITEMSTRUCT)lParam;

	switch (pdis->itemAction) {
	case ODA_DRAWENTIRE:
	case ODA_SELECT:
		{
			HBRUSH hbr,hbrOld;
			COLORREF cr;
			HPEN hpenOld;
			RECT rc;
			int nOldBkMode;
			COLORREF crOldTextColor;
			TCHAR szText[64];

			hbr=CreateSolidBrush(GetSysColor((pdis->itemState&ODS_DISABLED)!=0?
								COLOR_3DFACE:(pdis->itemState&ODS_SELECTED)==0?
												COLOR_WINDOW:COLOR_HIGHLIGHT));
			FillRect(pdis->hDC,&pdis->rcItem,hbr);
			DeleteObject(hbr);
			if (pdis->itemID==0)
				cr=(COLORREF)ComboBox_GetItemData(hwnd,0);
			else if (pdis->itemID<COLOR_TYPE_SYSCOLORFIRST)
				cr=ColorList[pdis->itemID-1].crColor;
			else
				cr=GetSysColor(ColorList[pdis->itemID-1].crColor);
			hbr=CreateSolidBrush(cr);
			hbrOld=SelectBrush(pdis->hDC,hbr);
			hpenOld=SelectPen(pdis->hDC,(HPEN)GetStockObject(BLACK_PEN));
			rc.left=pdis->rcItem.left+2;
			rc.top=pdis->rcItem.top+2;
			rc.bottom=pdis->rcItem.bottom-2;
			rc.right=rc.left+(rc.bottom-rc.top)*2;
			Rectangle(pdis->hDC,rc.left,rc.top,rc.right,rc.bottom);
			SelectPen(pdis->hDC,hpenOld);
			SelectBrush(pdis->hDC,hbrOld);
			DeleteBrush(hbr);
			nOldBkMode=SetBkMode(pdis->hDC,TRANSPARENT);
			crOldTextColor=SetTextColor(pdis->hDC,GetSysColor(
							(pdis->itemState&ODS_DISABLED)!=0?COLOR_GRAYTEXT:
								(pdis->itemState&ODS_SELECTED)==0?
										COLOR_WINDOWTEXT:COLOR_HIGHLIGHTTEXT));
			(void)ComboBox_GetLBText(hwnd,pdis->itemID,szText);
			rc.left=rc.right+2;
			rc.top=pdis->rcItem.top;
			rc.right=pdis->rcItem.right;
			rc.bottom=pdis->rcItem.bottom;
			DrawText(pdis->hDC,szText,-1,&rc,
										DT_LEFT | DT_SINGLELINE | DT_VCENTER);
			SetTextColor(pdis->hDC,crOldTextColor);
			SetBkMode(pdis->hDC,nOldBkMode);
			if ((pdis->itemState&ODS_FOCUS)==0)
				break;
		}
	case ODA_FOCUS:
		DrawFocusRect(pdis->hDC,&pdis->rcItem);
		break;
	}
	return TRUE;
}


BOOL ColorsComboBox_ChooseColor(HWND hDlg,int nID)
{
	HWND hwnd=GetDlgItem(hDlg,nID);
	COLORREF cr;

	cr=(COLORREF)ComboBox_GetItemData(hwnd,0);
	if (!ChooseColorDialog(hDlg,&cr))
		return FALSE;
	(void)ComboBox_SetItemData(hwnd,0,cr);
	if (ComboBox_GetCurSel(hwnd)!=0)
		(void)ComboBox_SetCurSel(hwnd,0);
	else
		InvalidateRect(hwnd,NULL,TRUE);
	return TRUE;
}
