/******************************************************************************
*                                                                             *
*    FileList.cpp                           Copyright(c) 2005-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <stdlib.h>
#include <windows.h>
#include <tchar.h>
#include "FileList.h"
#include "Util.h"




FileList::FileList()
{
	m_ppszList=NULL;
	m_ListLength=0;
	m_NumFiles=0;
}


FileList::~FileList()
{
	Clear();
}


void FileList::Clear()
{
	if (m_ppszList!=NULL) {
		for (int i=m_NumFiles-1;i>=0;i--)
			free(m_ppszList[i]);
		free(m_ppszList);
		m_ppszList=NULL;
		m_ListLength=0;
		m_NumFiles=0;
	}
}


/*
	t@Cǉ
*/
bool FileList::AddFile(LPCTSTR pszFileName)
{
	if (m_NumFiles==m_ListLength) {
		int NewLength=m_ListLength==0?16:m_ListLength*2;
		LPTSTR *ppszNewList;

		ppszNewList=(LPTSTR*)realloc(m_ppszList,NewLength*sizeof(LPTSTR));
		if (ppszNewList==NULL)
			return false;
		m_ppszList=ppszNewList;
		m_ListLength=NewLength;
	}
	LPTSTR pszFile=(LPTSTR)malloc((lstrlen(pszFileName)+1)*sizeof(TCHAR));
	if (pszFile==NULL)
		return false;
	lstrcpy(pszFile,pszFileName);
	m_ppszList[m_NumFiles++]=pszFile;
	return true;
}


/*
	fBNgt@Cǉ
*/
bool FileList::AddDirectoryFiles(LPCTSTR pszDirectory,LPCTSTR pszMasks,
															BOOL fFindSubDir)
{
	int First,Last;
	HANDLE hFind;
	WIN32_FIND_DATA wfd;
	LPCTSTR p;
	int i;
	TCHAR szMask[MAX_PATH];
	TCHAR szFileName[MAX_PATH];

	First=m_NumFiles;
	p=pszMasks;
	while (*p!='\0') {
		for (i=0;*p!='\0' && *p!=';';i++)
			szMask[i]=*p++;
		szMask[i]='\0';
		MergeFileName(szFileName,pszDirectory,szMask);
		hFind=FindFirstFile(szFileName,&wfd);
		if (hFind!=INVALID_HANDLE_VALUE) {
			do {
				if ((wfd.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)==0) {
					MergeFileName(szFileName,pszDirectory,wfd.cFileName);
					if (!AddFile(szFileName)) {
						FindClose(hFind);
						return false;
					}
				}
			} while (FindNextFile(hFind,&wfd));
			FindClose(hFind);
		}
		if (*p==';')
			p++;
	}
	Last=m_NumFiles-1;
	if (Last-First>0) {
		int j;
		bool fDeleted;

		SortSub(First,Last);
		// dĂt@C폜
		fDeleted=false;
		for (i=First;i<Last;) {
			for (j=i+1;j<=Last && lstrcmpi(m_ppszList[i],m_ppszList[j])==0;
																		j++) {
				free(m_ppszList[j]);
				m_ppszList[j]=NULL;
				fDeleted=true;
			}
			i=j;
		}
		if (fDeleted) {
			for (i=j=First+1;j<=Last;i++) {
				if (i!=j)
					m_ppszList[i]=m_ppszList[j];
				j++;
				while (j<=Last && m_ppszList[j]==NULL)
					j++;
			}
			m_NumFiles=i;
		}
	}
	// TufBNg
	if (fFindSubDir) {
		FileList Dirs;

		MergeFileName(szFileName,pszDirectory,TEXT("*.*"));
		hFind=FindFirstFile(szFileName,&wfd);
		if (hFind!=INVALID_HANDLE_VALUE) {
			do {
				if ((wfd.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)!=0
						&& lstrcmp(wfd.cFileName,TEXT("."))!=0
						&& lstrcmp(wfd.cFileName,TEXT(".."))!=0) {
					MergeFileName(szFileName,pszDirectory,wfd.cFileName);
					if (!Dirs.AddFile(szFileName)) {
						FindClose(hFind);
						return false;
					}
				}
			} while (FindNextFile(hFind,&wfd));
			FindClose(hFind);
		}
		if (Dirs.m_NumFiles>0) {
			Dirs.Sort();
			for (i=0;i<Dirs.m_NumFiles;i++) {
				if (!AddDirectoryFiles(Dirs.m_ppszList[i],pszMasks,TRUE))
					return false;
			}
		}
	}
	return true;
}


/*
	t@C擾
*/
bool FileList::GetFile(int Index,LPTSTR pszFileName) const
{
	if (Index<0 || Index>=m_NumFiles)
		return false;
	lstrcpy(pszFileName,m_ppszList[Index]);
	return true;
}


void FileList::SortSub(int nFirst,int nLast)
{
	int i=nFirst;
	int j=nLast;
	LPCTSTR p=m_ppszList[(nFirst+nLast)/2];
	LPTSTR pszTmp;

	while (i<=j) {
		while (lstrcmpi(m_ppszList[i],p)<0)
			i++;
		while (lstrcmpi(m_ppszList[j],p)>0)
			j--;
		if (i<=j) {
			pszTmp=m_ppszList[i];
			m_ppszList[i]=m_ppszList[j];
			m_ppszList[j]=pszTmp;
			i++;
			j--;
		}
	}
	if (j>nFirst)
		SortSub(nFirst,j);
	if (i<nLast)
		SortSub(i,nLast);
}


/*
	t@Cבւ
*/
void FileList::Sort()
{
	if (m_NumFiles>1)
		SortSub(0,m_NumFiles-1);
}
