/******************************************************************************
*                                                                             *
*    History.cpp                            Copyright(c) 2005-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <tchar.h>
#include "LafPlayer.h"
#include "History.h"




inline TCHAR GetAccelChar(int Index)
{
	return Index<10?Index+'0':Index-10+'A';
}


static void AddAmpersand(LPCTSTR pszSrc,LPTSTR pszDst)
{
	LPCTSTR p;
	LPTSTR q;

	p=pszSrc;
	q=pszDst;
	while (*p!='\0') {
#ifndef UNICODE
		if (*(p+1)!='\0' && IsDBCSLeadByteEx(CP_ACP,*p))
			*q++=*p++;
		else
#endif
		if (*p=='&')
			*q++='&';
		*q++=*p++;
	}
	*q='\0';
}


static void RemoveAmpersand(LPCTSTR pszSrc,LPTSTR pszDst)
{
	LPCTSTR p;
	LPTSTR q;

	p=pszSrc;
	q=pszDst;
	while (*p!='\0') {
#ifndef UNICODE
		if (*(p+1)!='\0' && IsDBCSLeadByteEx(CP_ACP,*p))
			*q++=*p++;
		else
#endif
		if (*p=='&' && *(p+1)=='&')
			p++;
		*q++=*p++;
	}
	*q='\0';
}


History::History()
{
	m_hmenuParent=NULL;
	m_hmenu=NULL;
	m_Max=10;
}


void History::SetMenu(HMENU hmenuParent,int Pos,int Command)
{
	m_hmenuParent=hmenuParent;
	m_MenuPos=Pos;
	m_hmenu=GetSubMenu(hmenuParent,Pos);
	m_Command=Command;
}


void History::SetMenu(HMENU hmenu,int Command)
{
	m_hmenuParent=NULL;
	m_hmenu=hmenu;
	m_Command=Command;
}


bool History::Add(LPCTSTR pszPath)
{
	int MenuItems,i;
	TCHAR szPath[MAX_PATH*2-1];
	TCHAR szText[MAX_PATH*2-1+4];

	if (m_Max<=0)
		return false;
	MenuItems=GetMenuItemCount(m_hmenu);
	if (MenuItems==0 && m_hmenuParent!=NULL)
		EnableMenuItem(m_hmenuParent,m_MenuPos,MF_BYPOSITION | MFS_ENABLED);
	AddAmpersand(pszPath,szPath);
	for (i=0;i<MenuItems;) {
		GetMenuString(m_hmenu,i,szText,numberof(szText),MF_BYPOSITION);
		if (lstrcmpi(szPath,szText+4)==0) {
			if (i==0) {
				// ɍŏɂ
				return true;
			} else {
				DeleteMenu(m_hmenu,i,MF_BYPOSITION);
				MenuItems--;
				continue;
			}
		}
		szText[1]=GetAccelChar(i+1);
		ModifyMenu(m_hmenu,i,MF_BYPOSITION | MFT_STRING,m_Command+i+1,szText);
		i++;
	}
	if (i>=m_Max)
		DeleteMenu(m_hmenu,m_Max-1,MF_BYPOSITION);
	wsprintf(szText,TEXT("&0: %s"),szPath);
	InsertMenu(m_hmenu,0,MF_BYPOSITION | MFT_STRING,m_Command,szText);
	return true;
}


bool History::Get(int Index,LPTSTR pszPath)
{
	TCHAR szText[MAX_PATH*2-1+4];

	GetMenuString(m_hmenu,Index,szText,numberof(szText),MF_BYPOSITION);
	RemoveAmpersand(szText+4,pszPath);
	return true;
}


bool History::Remove(int Index)
{
	int Items,i;

	Items=GetMenuItemCount(m_hmenu);
	if (Index<0 || Index>=Items)
		return false;
	DeleteMenu(m_hmenu,Index,MF_BYPOSITION);
	Items--;
	if (Items>0) {
		TCHAR szText[MAX_PATH*2-1+4];

		for (i=Index;i<Items;i++) {
			GetMenuString(m_hmenu,i,szText,numberof(szText),MF_BYPOSITION);
			szText[1]=GetAccelChar(i);
			ModifyMenu(m_hmenu,i,MF_BYPOSITION | MFT_STRING,m_Command+i,
																	szText);
		}
	} else if (m_hmenuParent!=NULL)
		EnableMenuItem(m_hmenuParent,m_MenuPos,MF_BYPOSITION | MFS_GRAYED);
	return true;
}


void History::Clear()
{
	int MenuItems,i;

	MenuItems=GetMenuItemCount(m_hmenu);
	if (MenuItems==0)
		return;
	for (i=0;i<MenuItems;i++)
		DeleteMenu(m_hmenu,0,MF_BYPOSITION);
	if (m_hmenuParent!=NULL)
		EnableMenuItem(m_hmenuParent,m_MenuPos,MF_BYPOSITION | MFS_GRAYED);
}


void History::SetMax(int Max)
{
	m_Max=Max;
	if (m_hmenu!=NULL) {
		if (Max==0) {
			Clear();
		} else {
			int Items,i;

			Items=GetMenuItemCount(m_hmenu);
			if (Items>Max) {
				for (i=Max;i<Items;i++)
					DeleteMenu(m_hmenu,i,MF_BYPOSITION);
			}
		}
	}
}


int History::GetCount()
{
	return GetMenuItemCount(m_hmenu);
}


bool History::Load(LPCTSTR pszFileName,LPCTSTR pszSection,LPCTSTR pszPrefix)
{
	Settings *pSettings;
	int Files,i;
	TCHAR szName[16];
	TCHAR szFileName[MAX_PATH],szMenu[MAX_PATH*2-1+4];

	if (m_Max==0)
		return true;
	pSettings=new Settings;
	if (!pSettings->Open(pszFileName,pszSection,Settings::OPEN_READ)) {
		delete pSettings;
		return false;
	}
	wsprintf(szName,TEXT("%sCount"),pszPrefix);
	if (!pSettings->ReadInt(szName,&Files) || Files<=0) {
		delete pSettings;
		return false;
	}
	if (Files>m_Max)
		Files=m_Max;
	for (i=0;i<Files;i++) {
		wsprintf(szName,TEXT("%s%d"),pszPrefix,i);
		if (!pSettings->ReadString(szName,szFileName,numberof(szFileName))
														|| szFileName[0]=='\0')
			break;
		wsprintf(szMenu,TEXT("&%c: "),GetAccelChar(i));
		AddAmpersand(szFileName,szMenu+4);
		AppendMenu(m_hmenu,MFT_STRING,m_Command+i,szMenu);
	}
	delete pSettings;
	if (m_hmenuParent!=NULL)
		EnableMenuItem(m_hmenuParent,m_MenuPos,
								MF_BYPOSITION | (i>0?MFS_ENABLED:MFS_GRAYED));
	return true;
}


bool History::Save(LPCTSTR pszFileName,LPCTSTR pszSection,LPCTSTR pszPrefix)
{
	Settings HistorySettings;
	int Items,i;
	TCHAR szPath[MAX_PATH],szName[16];

	if (!HistorySettings.Open(pszFileName,pszSection,Settings::OPEN_WRITE))
		return false;
	Items=GetMenuItemCount(m_hmenu);
	wsprintf(szName,TEXT("%sCount"),pszPrefix);
	HistorySettings.WriteInt(szName,Items);
	for (i=0;i<Items;i++) {
		Get(i,szPath);
		wsprintf(szName,TEXT("%s%d"),pszPrefix,i);
		HistorySettings.WriteString(szName,szPath);
	}
	return true;
}
