/******************************************************************************
*                                                                             *
*    Media.cpp                              Copyright(c) 2005-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <tchar.h>
#include "LafPlayer.h"
#include "Media.h"
#include "MediaMCI.h"
#include "MediaDirectShow.h"
#include "Util.h"
#include "resource.h"


#define MEDIA_WINDOW_CLASS TEXT("LafPlayer Media Window")
#define MEDIA_CANVAS_WINDOW_CLASS TEXT("LafPlayer Media Canvas Window")
#define FULLSCREEN_WINDOW_CLASS TEXT("LafPlayer FullScreen Window")

#define TIMER_ID 1
#define HIDE_CURSOR_TIME 1500

#define MediaPtr(hwnd) ((MediaPlayer*)GetWindowLongPtr(hwnd,0))


static const TBBUTTON tbbPlayBar[] = {
	{0,	CM_PLAY,		0,					BTNS_CHECK,		{0,0},	0,0},
	{1,	CM_PAUSE,		0,					BTNS_CHECK,		{0,0},	0,0},
	{2,	CM_STOP,		0,					BTNS_CHECK,		{0,0},	0,0},
	{0,	0,				TBSTATE_ENABLED,	BTNS_SEP,		{0,0},	0,0},
	{3,	CM_PREVFRAME,	0,					BTNS_BUTTON,	{0,0},	0,0},
	{4,	CM_NEXTFRAME,	0,					BTNS_BUTTON,	{0,0},	0,0},
	{5,	CM_BACKWARD,	0,					BTNS_CHECK,		{0,0},	0,0},
	{6,	CM_FORWARD,		0,					BTNS_CHECK,		{0,0},	0,0},
	{0,	0,				TBSTATE_ENABLED,	BTNS_SEP,		{0,0},	0,0},
	{7,	CM_REPEAT,		TBSTATE_ENABLED,	BTNS_CHECK,		{0,0},	0,0},
	{0,	0,				TBSTATE_ENABLED,	BTNS_SEP,		{0,0},	0,0},
	{8,	CM_MUTE,		TBSTATE_ENABLED,	BTNS_CHECK,		{0,0},	0,0},
};




Media::Media()
{
	m_hwnd=NULL;
	m_fOpened=false;
	m_MediaType=MEDIA_UNDEFINED;
	m_Mode=MEDIA_MODE_UNDEFINED;
	m_BackwardSpeed=2000;
	m_ForwardSpeed=2000;
	m_Volume=1000;
	m_fMute=false;
}


void Media::SendNotify(int Code,LPARAM Param)
{
	SendMessage(GetParent(m_hwnd),WM_MEDIA_NOTIFY,Code,Param);
}


bool Media::GetSourceSize(LPRECT prc)
{
	SIZE sz;

	if (!GetSourceSize(&sz))
		return false;
	prc->left=0;
	prc->top=0;
	prc->right=sz.cx;
	prc->bottom=sz.cy;
	return true;
}


bool Media::GetDestSize(LPRECT prc)
{
	SIZE sz;

	if (!GetDestSize(&sz))
		return false;
	prc->left=0;
	prc->top=0;
	prc->right=sz.cx;
	prc->bottom=sz.cy;
	return true;
}




void MediaPlayer::CalcDestSize(SIZE *psz)
{
	SIZE sz;

	m_pMedia->GetSourceSize(&sz);
	sz.cx*=m_Zoom;
	sz.cy*=m_Zoom;
	if (m_XAspect!=0 && m_YAspect!=0) {
		double SrcAspect,DstAspect;

		SrcAspect=(double)sz.cx/(double)sz.cy;
		DstAspect=(double)m_XAspect/(double)m_YAspect;
		if (SrcAspect<DstAspect)
			sz.cy=sz.cx*m_YAspect/m_XAspect;
		else if (SrcAspect>DstAspect)
			sz.cx=sz.cy*m_XAspect/m_YAspect;
	}
	sz.cx/=100;
	sz.cy/=100;
	if (sz.cx==0)
		sz.cx=1;
	if (sz.cy==0)
		sz.cy=1;
	*psz=sz;
}


HWND MediaPlayer::CreatePlayBar()
{
	HWND hwnd;
	int IconWidth,IconHeight;

	if (m_himlPlayBar!=NULL) {
		ImageList_GetIconSize(m_himlPlayBar,&IconWidth,&IconHeight);
	} else {
		IconWidth=16;
		IconHeight=15;
	}
	hwnd=CreateToolbarEx(m_hwnd,
			WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS |
				CCS_BOTTOM | CCS_NODIVIDER | TBSTYLE_TOOLTIPS | TBSTYLE_FLAT,0,
																	0,NULL,0,
					tbbPlayBar,numberof(tbbPlayBar),0,0,IconWidth,IconHeight,
															sizeof(TBBUTTON));
	if (hwnd==NULL)
		return NULL;
	SendMessage(hwnd,TB_SETIMAGELIST,0,(LPARAM)
		(m_himlPlayBar!=NULL?m_himlPlayBar:
			ImageList_LoadImage(GetResourceInstance(),
				MAKEINTRESOURCE(IDB_PLAYBAR),16,1,RGB(255,0,255),IMAGE_BITMAP,
														LR_CREATEDIBSECTION)));
	if (m_himlPlayBarHot!=NULL)
		SendMessage(hwnd,TB_SETHOTIMAGELIST,0,(LPARAM)m_himlPlayBarHot);
	if (m_himlPlayBarDisabled!=NULL)
		SendMessage(hwnd,TB_SETDISABLEDIMAGELIST,0,
												(LPARAM)m_himlPlayBarDisabled);
	RECT rc;
	SendMessage(hwnd,TB_GETITEMRECT,numberof(tbbPlayBar)-1,(LPARAM)&rc);
	if (m_pVolumeSlider==NULL) {
		m_pVolumeSlider=new VolumeSlider(hwnd,400,
						rc.right+4,rc.top,12,rc.bottom-rc.top,GetInstance());
		m_pVolumeSlider->SetVolume(500/*m_Volume*/);
	} else {
		SetParent(m_pVolumeSlider->GetHandle(),hwnd);
		MoveWindow(m_pVolumeSlider->GetHandle(),
								rc.right+4,rc.top,12,rc.bottom-rc.top,TRUE);
	}
	return hwnd;
}


bool MediaPlayer::CreateSeekBar()
{
	m_pSeekBar=new SeekBar(m_hwnd,0,
						WS_CHILD | WS_VISIBLE | WS_DISABLED | WS_CLIPSIBLINGS,
										0,0,0,16,GetWindowInstance(m_hwnd));
	return m_pSeekBar->GetHandle()!=NULL;
}


void MediaPlayer::OnCanvasPaint()
{
	PAINTSTRUCT ps;

	BeginPaint(m_hwndCanvas,&ps);
	if (ps.fErase) {
		HBRUSH hbr;

		hbr=CreateSolidBrush(m_fFullScreen?m_crFullScreenBg:m_crBackground);
		FillRect(ps.hdc,&ps.rcPaint,hbr);
		DeleteObject(hbr);
	}
	if (m_hbmLogo!=NULL && !m_fMediaChangeInterval
			&& GetMediaType()!=MEDIA_VIDEO) {
		RECT rc;
		BITMAP bm;
		int x,y;
		HDC hdcMemory;
		HBITMAP hbmOld;

		GetClientRect(m_hwndCanvas,&rc);
		GetObject(m_hbmLogo,sizeof(BITMAP),&bm);
		x=max((int)(rc.right-bm.bmWidth)/2,0);
		y=max((int)(rc.bottom-bm.bmHeight)/2,0);
		hdcMemory=CreateCompatibleDC(ps.hdc);
		hbmOld=(HBITMAP)SelectObject(hdcMemory,m_hbmLogo);
		BitBlt(ps.hdc,x,y,min(bm.bmWidth,rc.right),min(bm.bmHeight,rc.bottom),
														hdcMemory,0,0,SRCCOPY);
		SelectObject(hdcMemory,hbmOld);
		DeleteDC(hdcMemory);
	}
	EndPaint(m_hwndCanvas,&ps);
}


void MediaPlayer::OnCanvasSize(int Width,int Height)
{
	if (GetMediaType()==MEDIA_VIDEO) {
		SIZE sz;

		if (m_fFullScreen) {
			if (m_fFullScreenFitToDisplay) {
				m_pMedia->Move(0,0,Width,Height);
			} else {
				int ZoomWidth,ZoomHeight;

				m_pMedia->GetSourceSize(&sz);
				ZoomWidth=min((int)(sz.cx*Height/sz.cy),Width);
				ZoomHeight=min((int)(sz.cy*Width/sz.cx),Height);
				if (m_XAspect!=0 && m_YAspect!=0) {
					double SrcAspect,DstAspect;

					SrcAspect=(double)sz.cx/(double)sz.cy;
					DstAspect=(double)m_XAspect/(double)m_YAspect;
					if (SrcAspect<DstAspect)
						ZoomHeight=ZoomWidth*m_YAspect/m_XAspect;
					else if (SrcAspect>DstAspect)
						ZoomWidth=ZoomHeight*m_XAspect/m_YAspect;
				}
				m_pMedia->Move(max((Width-ZoomWidth)/2,0),
							max((Height-ZoomHeight)/2,0),ZoomWidth,ZoomHeight);
			}
			return;
		}
		// {EBhETCYɍ킹
		if (m_fFitToWindow) {
			int Zoom;

			m_pMedia->GetSourceSize(&sz);
			Zoom=min(Width*100/sz.cx,Height*100/sz.cy);
			if (Zoom<1)
				Zoom=1;
			if (Zoom!=m_Zoom) {
				m_Zoom=Zoom;
				CalcDestSize(&sz);
				m_pMedia->SetSize(sz.cx,sz.cy);
				SendNotify(MEDIAPLAYER_NOTIFY_ZOOMCHANGE);
			}
		}
		// fɕ\悤ɂ
		m_pMedia->GetDestSize(&sz);
		m_pMedia->Move(max((Width-sz.cx)/2,0),max((Height-sz.cy)/2,0),
																sz.cx,sz.cy);
	}
}


void MediaPlayer::OnLButtonDown()
{
	SendNotify(MEDIAPLAYER_NOTIFY_LBUTTONDOWN);
}


void MediaPlayer::OnRButtonDown()
{
	// j[̕\ɃJ[\Ȃ悤ɂ
	if (m_fFullScreen && m_fFullScreenHideCursor && m_fShowCursor)
		KillTimer(m_hwndCanvas,TIMER_ID);
	SendNotify(MEDIAPLAYER_NOTIFY_RBUTTONDOWN);
	if (m_fFullScreen && m_fFullScreenHideCursor) {
		SetCursor(LoadCursor(NULL,IDC_ARROW));
		m_fShowCursor=TRUE;
		SetTimer(m_hwndCanvas,TIMER_ID,HIDE_CURSOR_TIME,NULL);
	}
}


void MediaPlayer::OnDropFiles(HDROP hdrop)
{
	FORWARD_WM_DROPFILES(m_hwndParent,hdrop,SendMessage);
}


void MediaPlayer::OnNotifyMode(MediaMode Mode)
{
	switch (Mode) {
	case MEDIA_MODE_PLAY:
		SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_PLAY,TRUE);
		SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_STOP,TRUE);
		SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_PAUSE,TRUE);
		if (m_pMedia->GetMediaType()==MEDIA_VIDEO) {
			SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_PREVFRAME,FALSE);
			SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_NEXTFRAME,FALSE);
		}
		SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_BACKWARD,FALSE);
		//SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_BACKWARD,TRUE);
		SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_FORWARD,FALSE);
		//SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_FORWARD,TRUE);
		// XN[Z[o[L~߂
		if (m_fScreenSaverActive)
			SystemParametersInfo(SPI_SETSCREENSAVEACTIVE,FALSE,NULL,0);
		break;
	case MEDIA_MODE_STOP:
	case MEDIA_MODE_PAUSE:
		SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_PLAY,TRUE);
		SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_STOP,TRUE);
		SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_PAUSE,
													Mode==MEDIA_MODE_PAUSE);
		if (m_pMedia->GetMediaType()==MEDIA_VIDEO) {
			SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_PREVFRAME,TRUE);
			SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_NEXTFRAME,TRUE);
		}
		SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_BACKWARD,FALSE);
		//SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_BACKWARD,FALSE);
		SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_FORWARD,FALSE);
		//SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_FORWARD,FALSE);
		if (m_fScreenSaverActive)
			SystemParametersInfo(SPI_SETSCREENSAVEACTIVE,TRUE,NULL,0);
		break;
	default:
		return;
	}
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_PLAY,
											Mode==MEDIA_MODE_PLAY?TRUE:FALSE);
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_STOP,
											Mode==MEDIA_MODE_STOP?TRUE:FALSE);
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_PAUSE,
											Mode==MEDIA_MODE_PAUSE?TRUE:FALSE);
	SendMessage(m_hwndParent,WM_MEDIA_NOTIFY,MEDIA_NOTIFY_MODE,(LPARAM)Mode);
	// Ō܂ōĐꂽ肷
	if (Mode==MEDIA_MODE_STOP
			&& m_pMedia->GetPosition()==m_pMedia->GetEnd()) {
		// JԂݒ肳ĂāA[U[ɂ~łȂꍇ͌JԂ
		if (m_fRepeat && !m_fUserStop) {
			Home();
			Play();
		} else
			SendNotify(MEDIAPLAYER_NOTIFY_END);
	}
}


void MediaPlayer::OnNotifyPos(MediaPosition Pos)
{
	if (m_fShowSeekBar && !m_fSeeking)
		m_pSeekBar->SetPos(Pos);
	SendMessage(m_hwndParent,WM_MEDIA_NOTIFY,MEDIA_NOTIFY_POS,Pos);
}


LRESULT CALLBACK MediaPlayer::MediaCanvasWndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		SetWindowLongPtr(hwnd,0,
						(LONG_PTR)((LPCREATESTRUCT)lParam)->lpCreateParams);
		return 0;

	case WM_PAINT:
		MediaPtr(hwnd)->OnCanvasPaint();
		return 0;

	case WM_SIZE:
		MediaPtr(hwnd)->OnCanvasSize(LOWORD(lParam),HIWORD(lParam));
		return 0;

	case WM_LBUTTONDOWN:
		MediaPtr(hwnd)->OnLButtonDown();
		return 0;

	case WM_RBUTTONDOWN:
		MediaPtr(hwnd)->OnRButtonDown();
		return 0;

	case WM_DROPFILES:
		MediaPtr(hwnd)->OnDropFiles((HDROP)wParam);
		return 0;

	case WM_MOUSEMOVE:
		{
			MediaPlayer *pMedia=MediaPtr(hwnd);

			if (pMedia->m_fFullScreen && !pMedia->m_fShowCursor) {
				SetCursor(LoadCursor(NULL,IDC_ARROW));
				pMedia->m_fShowCursor=true;
				SetTimer(hwnd,TIMER_ID,HIDE_CURSOR_TIME,NULL);
			}
		}
		return 0;

	case WM_TIMER:
		SetCursor(NULL);
		MediaPtr(hwnd)->m_fShowCursor=false;
		KillTimer(hwnd,TIMER_ID);
		return 0;

	case WM_SETCURSOR:
		if (LOWORD(lParam)==HTCLIENT) {
			MediaPlayer *pMedia=MediaPtr(hwnd);

			if (!pMedia->m_fShowCursor) {
				SetCursor(NULL);
				return TRUE;
			}
		}
		break;

	case WM_MEDIA_NOTIFY:
		switch (wParam) {
		case MEDIA_NOTIFY_MODE:
			MediaPtr(hwnd)->OnNotifyMode((MediaMode)lParam);
			break;
		case MEDIA_NOTIFY_POS:
			MediaPtr(hwnd)->OnNotifyPos(lParam);
			break;
		case MEDIA_NOTIFY_LBUTTONDOWN:
			//MediaPtr(hwnd)->SendNotify(MEDIAPLAYER_NOTIFY_LBUTTONDOWN);
			MediaPtr(hwnd)->OnLButtonDown();
			break;
		case MEDIA_NOTIFY_RBUTTONDOWN:
			//MediaPtr(hwnd)->SendNotify(MEDIAPLAYER_NOTIFY_RBUTTONDOWN);
			MediaPtr(hwnd)->OnRButtonDown();
			break;
		}
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


bool MediaPlayer::CreateCanvas()
{
	m_hwndCanvas=CreateWindowEx(WS_EX_CLIENTEDGE,MEDIA_CANVAS_WINDOW_CLASS,
			NULL,WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN,0,0,0,0,m_hwnd,NULL,
											GetWindowInstance(m_hwnd),this);
	return m_hwndCanvas!=NULL;
}


bool MediaPlayer::CreateMedia()
{
	HINSTANCE hinst=GetWindowInstance(m_hwnd);

	if (m_pMedia!=NULL)
		delete m_pMedia;
	if (m_MediaDevice==MEDIA_DEVICE_MCI) {
		MCIMedia::Init(hinst);
		m_pMedia=new MCIMedia;
	} else {
		DirectShowMedia::Init(hinst);
		m_pMedia=new DirectShowMedia;
	}
	return m_pMedia->Create(m_hwndCanvas,0,hinst);
}


bool MediaPlayer::OnCreate(HWND hwnd)
{
	m_hwnd=hwnd;
	m_hwndPlayBar=CreatePlayBar();
	CreateSeekBar();
	CreateCanvas();
	CreateMedia();
	return true;
}


void MediaPlayer::OnSize(int Width,int Height)
{
	int Bottom;

	Bottom=Height;
	if (m_fShowPlayBar) {
		RECT rcPlayBar;

		SendMessage(m_hwndPlayBar,TB_AUTOSIZE,0,0);
		GetWindowRect(m_hwndPlayBar,&rcPlayBar);
		Bottom-=rcPlayBar.bottom-rcPlayBar.top;
	}
	if (m_fShowSeekBar) {
		RECT rcSeekBar;

		GetWindowRect(m_pSeekBar->GetHandle(),&rcSeekBar);
		Bottom-=rcSeekBar.bottom-rcSeekBar.top;
		MoveWindow(m_pSeekBar->GetHandle(),0,Bottom,
									Width,rcSeekBar.bottom-rcSeekBar.top,TRUE);
	}
	MoveWindow(m_hwndCanvas,0,0,Width,Bottom,TRUE);
}


void MediaPlayer::OnHScroll(int Type)
{
	if (IsOpen()) {
		int Pos;

		if (!m_fSeeking) {
			m_SeekOldMode=m_pMedia->GetMode();
			m_fSeeking=true;
			if (m_SeekOldMode==MEDIA_MODE_PLAY)
				m_pMedia->Pause();
		}
		Pos=m_pSeekBar->GetPos();
		m_pMedia->SetPosition(Pos);
		if (Type==SeekBar::TRACK_END) {
			if (m_SeekOldMode==MEDIA_MODE_PLAY)
				m_pMedia->Play();
			m_fSeeking=false;
		}
	}
}


LRESULT MediaPlayer::OnNotify(LPNMHDR pnmhdr)
{
	switch (pnmhdr->code) {
	case NM_RCLICK:
		if (pnmhdr->hwndFrom==m_hwndPlayBar)
			SendNotify(MEDIAPLAYER_NOTIFY_PLAYBAR_RBUTTONDOWN);
		else if (pnmhdr->hwndFrom==m_pSeekBar->GetHandle())
			SendNotify(MEDIAPLAYER_NOTIFY_SEEKBAR_RBUTTONDOWN);
		break;
	case TTN_NEEDTEXT:
		{
			LPTOOLTIPTEXT pttt=(LPTOOLTIPTEXT)pnmhdr;

			pttt->lpszText=(LPTSTR)(IDS_SHORTCMBASE+pttt->hdr.idFrom);
			pttt->hinst=GetWindowInstance(m_hwnd);
		}
		break;
	}
	return 0;
}


void MediaPlayer::OnCommand(int nID,HWND hwndCtrl,UINT /*uNotifyCode*/)
{
	switch (nID) {
	case CM_PLAY:
		Play();
		break;
	case CM_PAUSE:
		Pause();
		break;
	case CM_PLAY_PAUSE:
		if (m_pMedia->GetMode()!=MEDIA_MODE_PLAY)
			Play();
		else
			Pause();
		break;
	case CM_STOP:
		Stop();
		// ~ɂ܂~{^ƃ`FbNÔ...
		SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_STOP,TRUE);
		break;
	case CM_PREVFRAME:
		PrevFrame();
		break;
	case CM_NEXTFRAME:
		NextFrame();
		break;
	case CM_BACKWARD:
		if (!Backward())
			SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_BACKWARD,FALSE);
		break;
	case CM_FORWARD:
		if (!Forward())
			SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_FORWARD,FALSE);
		break;
	case CM_HOME:
		Home();
		break;
	case CM_END:
		End();
		break;
	case CM_REPEAT:
		m_fRepeat=!m_fRepeat;
		SendNotify(MEDIAPLAYER_NOTIFY_REPEAT);
		break;
	case CM_MUTE:
		Mute(SendMessage(m_hwndPlayBar,TB_ISBUTTONCHECKED,CM_MUTE,0)!=0);
		break;
	default:
		if (hwndCtrl==m_pVolumeSlider->GetHandle())
			SetVolume(m_pVolumeSlider->GetVolume());
	}
}


void MediaPlayer::OnDestroy()
{
	delete m_pMedia;
	m_pMedia=NULL;
	if (m_pszFileName!=NULL) {
		delete [] m_pszFileName;
		m_pszFileName;
	}
	m_hwnd=NULL;
	m_hwndPlayBar=NULL;
	m_hwndCanvas=NULL;
	// XN[Z[o[𖳌ɂĂL
	if (m_fScreenSaverActive)
		SystemParametersInfo(SPI_SETSCREENSAVEACTIVE,TRUE,NULL,0);
}


LRESULT CALLBACK MediaPlayer::MediaWndProc(HWND hwnd,UINT uMsg,WPARAM wParam,
																LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			MediaPlayer *pMedia=
						(MediaPlayer*)((LPCREATESTRUCT)lParam)->lpCreateParams;

			SetWindowLongPtr(hwnd,0,(LONG_PTR)pMedia);
			return pMedia->OnCreate(hwnd)?0:-1;
		}

	case WM_SIZE:
		MediaPtr(hwnd)->OnSize(LOWORD(lParam),HIWORD(lParam));
		return 0;

	case WM_HSCROLL:
		MediaPtr(hwnd)->OnHScroll(LOWORD(wParam));
		return 0;

	case WM_NOTIFY:
		return MediaPtr(hwnd)->OnNotify((LPNMHDR)lParam);

	case WM_COMMAND:
		MediaPtr(hwnd)->OnCommand(GET_WM_COMMAND_ID(wParam,lParam),
											GET_WM_COMMAND_HWND(wParam,lParam),
											GET_WM_COMMAND_CMD(wParam,lParam));
		return 0;

	case WM_DESTROY:
		MediaPtr(hwnd)->OnDestroy();
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


void MediaPlayer::SendNotify(int Code)
{
	FORWARD_WM_COMMAND(m_hwndParent,GetWindowID(m_hwnd),m_hwnd,Code,
																SendMessage);
}


bool MediaPlayer::Init(HINSTANCE hinst)
{
	WNDCLASS wc;

	wc.style=0;
	wc.lpfnWndProc=MediaWndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=sizeof(MediaPlayer*);
	wc.hInstance=hinst;
	wc.hIcon=NULL;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=(HBRUSH)(COLOR_3DFACE+1);
	wc.lpszMenuName=NULL;
	wc.lpszClassName=MEDIA_WINDOW_CLASS;
	if (RegisterClass(&wc)==0)
		return false;
	wc.style=CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc=MediaCanvasWndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=sizeof(MediaPlayer*);
	wc.hInstance=hinst;
	wc.hIcon=NULL;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=NULL;
	wc.lpszMenuName=NULL;
	wc.lpszClassName=MEDIA_CANVAS_WINDOW_CLASS;
	if (RegisterClass(&wc)==0)
		return false;
	if (!VolumeSlider::Init(hinst) || !SeekBar::Init(hinst))
		return false;
	return true;
}


MediaPlayer::MediaPlayer()
{
	m_hwnd=NULL;
	m_hwndParent=NULL;
	m_MediaDevice=MEDIA_DEVICE_DIRECTSHOW;
	m_pMedia=NULL;
	m_pszFileName=NULL;
	m_hwndPlayBar=NULL;
	m_himlPlayBar=NULL;
	m_himlPlayBarHot=NULL;
	m_himlPlayBarDisabled=NULL;
	m_pVolumeSlider=NULL;
	m_pSeekBar=NULL;
	m_hwndCanvas=NULL;
	m_fShowPlayBar=true;
	m_fShowSeekBar=true;
	m_crBackground=RGB(0,0,0);
	m_crFullScreenBg=RGB(0,0,0);
	m_hbmLogo=LoadBitmap(GetResourceInstance(),MAKEINTRESOURCE(IDB_LOGO));
	m_Zoom=100;
	m_XAspect=0;
	m_YAspect=0;
	m_ForwardSpeed=2000;
	m_BackwardSpeed=2000;
	m_fRepeat=false;
	m_fFullScreen=false;
	m_hwndFullScreen=false;
	m_fFullScreenFitToDisplay=false;
	m_fFullScreenHideCursor=false;
	m_fShowCursor=true;
	m_fFitToWindow=false;
	m_fSeeking=false;
	m_fMediaChangeInterval=false;
}


MediaPlayer::~MediaPlayer()
{
	if (m_hwnd!=NULL)
		DestroyWindow(m_hwnd);
	if (m_hbmLogo!=NULL)
		DeleteBitmap(m_hbmLogo);
	if (m_hwndFullScreen!=NULL)
		DestroyWindow(m_hwndFullScreen);
	delete m_pVolumeSlider;
	delete m_pSeekBar;
}


bool MediaPlayer::Create(HWND hwndParent,int ID,HINSTANCE hinst)
{
	m_hwndParent=hwndParent;
	return CreateWindowEx(0,MEDIA_WINDOW_CLASS,NULL,
					WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN,
					CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,
									hwndParent,(HMENU)ID,hinst,this)!=NULL;
}


bool MediaPlayer::SetMediaDevice(MediaDeviceType Device)
{
	if (m_MediaDevice!=Device) {
		LPTSTR pszFileName;

		if (IsOpen()) {
			pszFileName=new TCHAR[lstrlen(m_pszFileName)+1];
			lstrcpy(pszFileName,m_pszFileName);
		} else
			pszFileName=NULL;
		m_MediaDevice=Device;
		if (m_pMedia!=NULL) {
			if (!CreateMedia())
				return false;
			if (pszFileName!=NULL) {
				Open(pszFileName);
				delete [] pszFileName;
			}
		}
	}
	return true;
}


MediaType MediaPlayer::GetMediaType() const
{
	if (m_pMedia==NULL)
		return MEDIA_UNDEFINED;
	return m_pMedia->GetMediaType();
}


bool MediaPlayer::IsOpen() const
{
	return m_pMedia!=NULL && m_pMedia->IsOpen();
}


bool MediaPlayer::Open(LPCTSTR pszFileName)
{
	if (IsOpen()) {
		m_fMediaChangeInterval=true;
		Close();
		UpdateWindow(m_hwndCanvas);
	}
	if (m_pMedia==NULL) {
		if (!CreateMedia()) {
			m_fMediaChangeInterval=false;
			return false;
		}
	}
	if (!m_pMedia->Open(pszFileName)) {
		m_fMediaChangeInterval=false;
		return false;
	}
	m_fMediaChangeInterval=false;
	m_pszFileName=new TCHAR[lstrlen(pszFileName)+1];
	lstrcpy(m_pszFileName,pszFileName);
	m_Zoom=100;
	m_XAspect=0;
	m_YAspect=0;
	/*if (m_fFullScreen || m_fFitToWindow) */{
		RECT rc;

		GetClientRect(m_hwndCanvas,&rc);
		OnCanvasSize(rc.right,rc.bottom);
	}
	m_pMedia->SetVolume(m_pVolumeSlider->GetVolume());
	m_pMedia->Mute(SendMessage(m_hwndPlayBar,TB_ISBUTTONCHECKED,CM_MUTE,0)!=0);
	m_pMedia->SetBackwardSpeed(m_BackwardSpeed);
	m_pMedia->SetForwardSpeed(m_ForwardSpeed);
	// V[No[̐ݒ
	MediaPosition Start=m_pMedia->GetStart();
	m_pSeekBar->SetRange(Start,m_pMedia->GetEnd());
	if (m_fShowSeekBar)
		m_pSeekBar->SetPos(Start);
	EnableWindow(m_pSeekBar->GetHandle(),TRUE);
	// vCo[̐ݒ
	SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_PLAY,TRUE);
	if (GetMediaType()==MEDIA_VIDEO) {
		SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_PREVFRAME,TRUE);
		SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_NEXTFRAME,TRUE);
	}
	SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_BACKWARD,TRUE/*CanBackward()*/);
	SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,CM_FORWARD,TRUE);
	//m_pVolumeSlider->SetVolume(m_pMedia->GetVolume());
	// XN[Z[o[L擾
	if (!SystemParametersInfo(SPI_GETSCREENSAVEACTIVE,0,&m_fScreenSaverActive,
																			0))
		m_fScreenSaverActive=FALSE;
	return TRUE;
}


void MediaPlayer::Close()
{
	if (IsOpen()) {
		//m_pMedia->Close();
		delete m_pMedia;
		m_pMedia=NULL;
		delete [] m_pszFileName;
		m_pszFileName=NULL;
		// vCo[̃{^𖳌ɂ
		for (int i=0;i<8;i++) {
			if ((tbbPlayBar[i].fsStyle&BTNS_CHECK)!=0)
				SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,
												tbbPlayBar[i].idCommand,FALSE);
			if ((tbbPlayBar[i].fsStyle&BTNS_SEP)==0)
				SendMessage(m_hwndPlayBar,TB_ENABLEBUTTON,
												tbbPlayBar[i].idCommand,FALSE);
		}
		m_pSeekBar->SetRange(0,100);
		m_pSeekBar->SetPos(0);
		EnableWindow(m_pSeekBar->GetHandle(),FALSE);
		// S\悤ɂ
		InvalidateRect(m_hwndCanvas,NULL,TRUE);
		// XN[Z[o[𖳌ɂĂL
		if (m_fScreenSaverActive) {
			SystemParametersInfo(SPI_SETSCREENSAVEACTIVE,TRUE,NULL,0);
			m_fScreenSaverActive=FALSE;
		}
	}
}


bool MediaPlayer::Play()
{
	if (!IsOpen())
		return false;
	m_fUserStop=FALSE;
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_BACKWARD,FALSE);
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_FORWARD,FALSE);
	if (m_pMedia->GetPosition()==m_pMedia->GetEnd())
		m_pMedia->Home();
	return m_pMedia->Play();
}


bool MediaPlayer::Pause()
{
	if (!IsOpen())
		return false;
	m_pMedia->Pause();
	return true;
}


bool MediaPlayer::Stop()
{
	if (!IsOpen())
		return false;
	m_fUserStop=TRUE;
	return m_pMedia->Stop();
}


bool MediaPlayer::PrevFrame()
{
	if (!IsOpen())
		return false;
	return m_pMedia->PrevFrame();
}


bool MediaPlayer::NextFrame()
{
	if (!IsOpen())
		return false;
	return m_pMedia->NextFrame();
}


bool MediaPlayer::Backward()
{
	if (!IsOpen())
		return false;
	if (!m_pMedia->Backward())
		return false;
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_PLAY,FALSE);
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_FORWARD,FALSE);
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_BACKWARD,TRUE);
	return true;
}


bool MediaPlayer::Forward()
{
	if (!IsOpen())
		return false;
	if (!m_pMedia->Forward())
		return false;
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_PLAY,FALSE);
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_BACKWARD,FALSE);
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_FORWARD,TRUE);
	return true;
}


bool MediaPlayer::SetBackwardSpeed(int Speed)
{
	if (m_pMedia!=NULL) {
		if (!m_pMedia->SetBackwardSpeed(Speed))
			return false;
	}
	m_BackwardSpeed=Speed;
	return true;
}


bool MediaPlayer::SetForwardSpeed(int Speed)
{
	if (m_pMedia!=NULL) {
		if (!m_pMedia->SetForwardSpeed(Speed))
			return false;
	}
	m_ForwardSpeed=Speed;
	return true;
}


bool MediaPlayer::Home()
{
	if (!IsOpen())
		return false;
	return m_pMedia->Home();
}


bool MediaPlayer::End()
{
	if (!IsOpen())
		return false;
	return m_pMedia->End();
}


bool MediaPlayer::SetZoom(int Zoom)
{
	if (Zoom<=0)
		return false;
	m_Zoom=Zoom;
	if (IsOpen() && !m_fFullScreen) {
		SIZE sz;
		RECT rc;

		CalcDestSize(&sz);
		m_pMedia->SetSize(sz.cx,sz.cy);
		GetClientRect(m_hwndCanvas,&rc);
		OnCanvasSize(rc.right,rc.bottom);
	}
	return true;
}


bool MediaPlayer::SetAspectRatio(int XAspect,int YAspect)
{
	if (XAspect<0 || YAspect<0)
		return false;
	m_XAspect=XAspect;
	m_YAspect=YAspect;
	if (IsOpen()) {
		SIZE sz;
		RECT rc;

		CalcDestSize(&sz);
		m_pMedia->SetSize(sz.cx,sz.cy);
		GetClientRect(m_hwndCanvas,&rc);
		OnCanvasSize(rc.right,rc.bottom);
	}
	return true;
}


void MediaPlayer::SetRepeat(bool fRepeat)
{
	m_fRepeat=fRepeat;
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_REPEAT,m_fRepeat);
}


unsigned int MediaPlayer::GetVolume() const
{
	if (m_pMedia==NULL)
		return m_pVolumeSlider->GetVolume();
	return m_pMedia->GetVolume();
}


bool MediaPlayer::SetVolume(unsigned int Volume)
{
	if (m_pMedia!=NULL) {
		bool fMute=m_pMedia->IsMute();

		if (!m_pMedia->SetVolume(Volume))
			return false;
		if (fMute)
			SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_MUTE,FALSE);
	}
	m_pVolumeSlider->SetVolume(Volume);
	return true;
}


bool MediaPlayer::Mute(bool fMute)
{
	m_pMedia->Mute(fMute);
	SendMessage(m_hwndPlayBar,TB_CHECKBUTTON,CM_MUTE,m_pMedia->IsMute());
	return true;
}


bool MediaPlayer::IsMute() const
{
	return m_pMedia->IsMute();
}


bool MediaPlayer::GetSourceSize(LPSIZE psz)
{
	return m_pMedia->GetSourceSize(psz);
}


bool MediaPlayer::GetSourceSize(LPRECT prc)
{
	return m_pMedia->GetSourceSize(prc);
}


bool MediaPlayer::GetDestSize(LPSIZE psz)
{
	return m_pMedia->GetDestSize(psz);
}


bool MediaPlayer::GetDestSize(LPRECT prc)
{
	return m_pMedia->GetDestSize(prc);
}


MediaPosition MediaPlayer::GetLength()
{
	return m_pMedia->GetLength();
}


MediaPosition MediaPlayer::GetStart()
{
	return m_pMedia->GetStart();
}


MediaPosition MediaPlayer::GetEnd()
{
	return m_pMedia->GetEnd();
}


MediaPosition MediaPlayer::GetPosition()
{
	return m_pMedia->GetPosition();
}


MediaMode MediaPlayer::GetMode() const
{
	return m_pMedia->GetMode();
}


bool MediaPlayer::GetFileName(LPTSTR pszFileName,int MaxLength) const
{
	if (!IsOpen())
		return false;
	lstrcpyn(pszFileName,m_pszFileName,MaxLength);
	return true;
}


bool MediaPlayer::HasAudio()
{
	return m_pMedia->HasAudio();
}


LRESULT CALLBACK MediaPlayer::FullScreenWndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			MediaPlayer *pMedia=
						(MediaPlayer*)((LPCREATESTRUCT)lParam)->lpCreateParams;

			SetWindowLongPtr(hwnd,0,(LONG_PTR)pMedia);
			return pMedia->OnFullScreenCreate(hwnd)?0:-1;
		}

	case WM_SIZE:
		{
			MediaPlayer *pMedia=MediaPtr(hwnd);

			MoveWindow(pMedia->m_hwnd,0,0,LOWORD(lParam),HIWORD(lParam),TRUE);
		}
		return 0;

	/*
	case WM_HSCROLL:
		MediaPtr(hwnd)->OnHScroll(LOWORD(wParam));
		return 0;
	*/

	case WM_KEYDOWN:
		if (wParam==VK_ESCAPE)
			SendMessage(hwnd,WM_CLOSE,0,0);
		return 0;

	/*
	case WM_NOTIFY:
		return MediaPtr(hwnd)->OnNotify((LPNMHDR)lParam);

	case WM_COMMAND:
		MediaPtr(hwnd)->OnCommand(GET_WM_COMMAND_ID(wParam,lParam),
											GET_WM_COMMAND_HWND(wParam,lParam),
											GET_WM_COMMAND_CMD(wParam,lParam));
		return 0;
	*/

	case WM_CLOSE:
		MediaPtr(hwnd)->OnFullScreenClose();
		break;

	case WM_DESTROY:
		MediaPtr(hwnd)->OnFullScreenDestroy();
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


bool MediaPlayer::FullScreen(bool fFullScreen)
{
	if (m_fFullScreen==fFullScreen)
		return true;
	if (fFullScreen) {
		static BOOL fRegisterd=FALSE;

		if (!fRegisterd) {
			WNDCLASS wc;

			wc.style=0;
			wc.lpfnWndProc=FullScreenWndProc;
			wc.cbClsExtra=0;
			wc.cbWndExtra=sizeof(Media*);
			wc.hInstance=GetWindowInstance(m_hwnd);
			wc.hIcon=NULL;
			wc.hCursor=LoadCursor(NULL,IDC_ARROW);
			wc.hbrBackground=(HBRUSH)(COLOR_3DFACE+1);
			wc.lpszMenuName=NULL;
			wc.lpszClassName=FULLSCREEN_WINDOW_CLASS;
			if (RegisterClass(&wc)==0)
				return false;
			fRegisterd=TRUE;
		}
		CreateWindowEx(WS_EX_TOPMOST,FULLSCREEN_WINDOW_CLASS,NULL,
			WS_POPUP | WS_VISIBLE | WS_CLIPCHILDREN,
			0,0,GetSystemMetrics(SM_CXSCREEN),GetSystemMetrics(SM_CYSCREEN),
							m_hwndParent,NULL,GetWindowInstance(m_hwnd),this);
	} else {
		SendMessage(m_hwndFullScreen,WM_CLOSE,0,0);
	}
	return true;
}


bool MediaPlayer::OnFullScreenCreate(HWND hwnd)
{
	m_hwndFullScreen=hwnd;
	m_fFullScreen=true;
	m_OldZoom=m_Zoom;
	//SetParent(m_hwndCanvas,m_hwndFullScreen);
	SetWindowLong(m_hwndCanvas,GWL_EXSTYLE,
							GetWindowExStyle(m_hwndCanvas)^WS_EX_CLIENTEDGE);
	//SetParent(m_hwndPlayBar,m_hwndFullScreen);
	//SetParent(m_pSeekBar->GetHandle(),m_hwndFullScreen);
	SetParent(m_hwnd,m_hwndFullScreen);
	if (m_fFullScreenHideCursor)
		m_fShowCursor=false;
	return true;
}


void MediaPlayer::OnFullScreenClose()
{
	RECT rc;

	if (m_fFullScreenHideCursor && m_fShowCursor)
		KillTimer(m_hwndCanvas,TIMER_ID);
	SetWindowLong(m_hwndCanvas,GWL_EXSTYLE,
							GetWindowExStyle(m_hwndCanvas) | WS_EX_CLIENTEDGE);
	//SetParent(m_hwndCanvas,m_hwnd);
	//SetParent(m_hwndPlayBar,m_hwnd);
	//SetParent(m_pSeekBar->GetHandle(),m_hwnd);
	SetParent(m_hwnd,m_hwndParent);
	m_fFullScreen=false;
	if (!m_fFitToWindow)
		SetZoom(m_OldZoom);
	GetClientRect(m_hwndParent,&rc);
	SendMessage(m_hwndParent,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom));
}


void MediaPlayer::OnFullScreenDestroy()
{
	m_hwndFullScreen=NULL;
	m_fShowCursor=true;
	SendNotify(MEDIAPLAYER_NOTIFY_FULLSCREENEND);
}


void MediaPlayer::FitToWindow(bool fFit)
{
	if (m_fFitToWindow!=fFit) {
		m_fFitToWindow=fFit;
		if (m_fFitToWindow && !m_fFullScreen && GetMediaType()==MEDIA_VIDEO) {
			RECT rc;

			GetClientRect(m_hwndCanvas,&rc);
			OnCanvasSize(rc.right,rc.bottom);
		}
	}
}


void MediaPlayer::FullScreenFitToDisplay(bool fFit)
{
	if (m_fFullScreenFitToDisplay!=fFit) {
		m_fFullScreenFitToDisplay=fFit;
		if (m_fFullScreen && GetMediaType()==MEDIA_VIDEO) {
			RECT rc;

			GetClientRect(m_hwndCanvas,&rc);
			OnCanvasSize(rc.right,rc.bottom);
		}
	}
}


void MediaPlayer::FullScreenHideCursor(bool fHide)
{
	m_fFullScreenHideCursor=fHide;
}


void MediaPlayer::AdjustRect(LPRECT prc)
{
	int cxEdge,cyEdge;
	RECT rc;

	cxEdge=GetSystemMetrics(SM_CXEDGE);
	cyEdge=GetSystemMetrics(SM_CYEDGE);
	prc->left-=cxEdge;
	prc->top-=cyEdge;
	prc->right+=cxEdge;
	prc->bottom+=cyEdge;
	if (m_fShowSeekBar) {
		GetWindowRect(m_pSeekBar->GetHandle()/*m_hwndSeekBar*/,&rc);
		prc->bottom+=rc.bottom-rc.top;
	}
	if (m_fShowPlayBar) {
		GetWindowRect(m_hwndPlayBar,&rc);
		prc->bottom+=rc.bottom-rc.top;
	}
}


void MediaPlayer::ShowPlayBar(bool fShow)
{
	if (m_fShowPlayBar!=fShow) {
		RECT rc;

		m_fShowPlayBar=fShow;
		GetClientRect(m_hwnd,&rc);
		OnSize(rc.right,rc.bottom);
		ShowWindow(m_hwndPlayBar,m_fShowPlayBar?SW_SHOW:SW_HIDE);
	}
}


void MediaPlayer::ShowSeekBar(bool fShow)
{
	if (m_fShowSeekBar!=fShow) {
		RECT rc;

		m_fShowSeekBar=fShow;
		GetClientRect(m_hwnd,&rc);
		OnSize(rc.right,rc.bottom);
		if (m_fShowSeekBar && IsOpen())
			m_pSeekBar->SetPos(m_pMedia->GetPosition());
		ShowWindow(m_pSeekBar->GetHandle(),m_fShowSeekBar?SW_SHOW:SW_HIDE);
	}
}


void MediaPlayer::ShowTipHelp(bool fShow)
{
	ShowToolbarTipHelp(m_hwndPlayBar,fShow);
}


void MediaPlayer::SetBackgroundColor(COLORREF cr)
{
	if (cr!=m_crBackground) {
		m_crBackground=cr;
		if (!m_fFullScreen)
			InvalidateRect(m_hwndCanvas,NULL,TRUE);
	}
}


void MediaPlayer::SetFullScreenBgColor(COLORREF cr)
{
	if (cr!=m_crFullScreenBg) {
		m_crFullScreenBg=cr;
		if (m_fFullScreen)
			InvalidateRect(m_hwndCanvas,NULL,TRUE);
	}
}


void MediaPlayer::SetTipColor(COLORREF crText,COLORREF crBack)
{
	HWND hwndToolTips;

	hwndToolTips=(HWND)SendMessage(m_hwndPlayBar,TB_GETTOOLTIPS,0,0);
	SendMessage(hwndToolTips,TTM_SETTIPTEXTCOLOR,crText,0);
	SendMessage(hwndToolTips,TTM_SETTIPBKCOLOR,crBack,0);
}


void MediaPlayer::SetVolumeColor(COLORREF crColor)
{
	m_pVolumeSlider->SetColor(crColor);
}


void MediaPlayer::SetSeekBarBarColor(COLORREF crColor)
{
	m_pSeekBar->SetBarColor(crColor);
}


bool MediaPlayer::SetPlayBarImage(HIMAGELIST himl,HIMAGELIST himlHot,
													HIMAGELIST himlDisabled)
{
	m_himlPlayBar=himl;
	m_himlPlayBarHot=himlHot;
	m_himlPlayBarDisabled=himlDisabled;
	if (m_hwndPlayBar!=NULL) {
		HWND hwnd;
		int i;
		RECT rc;

		hwnd=CreatePlayBar();
		if (hwnd==NULL)
			return false;
		for (i=0;i<numberof(tbbPlayBar);i++)
			SendMessage(hwnd,TB_SETSTATE,tbbPlayBar[i].idCommand,
				SendMessage(m_hwndPlayBar,TB_GETSTATE,tbbPlayBar[i].idCommand,
																		0));
		DestroyWindow(m_hwndPlayBar);
		m_hwndPlayBar=hwnd;
		GetClientRect(m_hwnd,&rc);
		OnSize(rc.right,rc.bottom);
	}
	return true;
}


void MediaPlayer::GetCanvasRect(RECT *prc)
{
	GetClientRect(m_hwndCanvas,prc);
	MapWindowRect(m_hwndCanvas,NULL,prc);
}


void MediaPlayer::SetLogo(HBITMAP hbm)
{
	if (m_hbmLogo!=NULL)
		DeleteBitmap(m_hbmLogo);
	if (hbm!=NULL)
		m_hbmLogo=hbm;
	else
		m_hbmLogo=LoadBitmap(GetResourceInstance(),MAKEINTRESOURCE(IDB_LOGO));
	if (!m_fMediaChangeInterval && GetMediaType()!=MEDIA_VIDEO)
		InvalidateRect(m_hwndCanvas,NULL,TRUE);
}
