/******************************************************************************
*                                                                             *
*    Media.h                                Copyright(c) 2005-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef MEDIA_H
#define MEDIA_H


#include <commctrl.h>
#include "SeekBar.h"
#include "Volume.h"


enum MediaType {
	MEDIA_UNDEFINED=-1,
	MEDIA_SOUND,
	MEDIA_VIDEO
};

enum MediaMode {
	MEDIA_MODE_UNDEFINED,
	MEDIA_MODE_READY,
	MEDIA_MODE_STOP,
	MEDIA_MODE_PLAY,
	MEDIA_MODE_PAUSE
};

typedef long MediaPosition;

#define WM_MEDIA_NOTIFY WM_APP
#define MEDIA_NOTIFY_MODE			1
#define MEDIA_NOTIFY_POS			2
#define MEDIA_NOTIFY_LBUTTONDOWN	3
#define MEDIA_NOTIFY_RBUTTONDOWN	4

class Media {
protected:
	HWND m_hwnd;
	bool m_fOpened;
	MediaType m_MediaType;
	MediaMode m_Mode;
	int m_BackwardSpeed;
	int m_ForwardSpeed;
	unsigned int m_Volume;
	bool m_fMute;
	void SendNotify(int Code,LPARAM Param);
public:
	Media();
	virtual ~Media() {}
	virtual bool Create(HWND hwndParent,int ID,HINSTANCE hinst)=0;
	virtual void Destroy()=0;
	HWND GetHandle() { return m_hwnd; }
	MediaType GetMediaType() const { return m_MediaType; }
	bool IsOpen() const { return m_fOpened; }
	virtual bool Open(LPCTSTR pszFileName)=0;
	virtual void Close()=0;
	virtual bool Play()=0;
	virtual bool Pause()=0;
	virtual bool Stop()=0;
	virtual bool PrevFrame() { return false; }
	virtual bool NextFrame() { return false; }
	virtual bool Backward()=0;
	virtual bool Forward()=0;
	virtual bool SetBackwardSpeed(int Speed)=0;
	virtual bool SetForwardSpeed(int Speed)=0;
	virtual bool Home()=0;
	virtual bool End()=0;
	unsigned int GetVolume() const { return m_Volume; }
	virtual bool SetVolume(unsigned int Volume)=0;
	virtual bool Mute(bool fMute)=0;
	bool IsMute() const { return m_fMute; }
	virtual bool GetSourceSize(LPSIZE psz)=0;
	bool GetSourceSize(LPRECT prc);
	virtual bool GetDestSize(LPSIZE psz)=0;
	bool GetDestSize(LPRECT prc);
	virtual MediaPosition GetLength()=0;
	virtual MediaPosition GetStart()=0;
	virtual MediaPosition GetEnd()=0;
	virtual MediaPosition GetPosition()=0;
	virtual bool SetPosition(MediaPosition Position)=0;
	virtual bool HasAudio()=0;
	//bool CanBackward();
	MediaMode GetMode() const { return m_Mode; }
	virtual void Move(int x,int y,int Width,int Height)=0;
	virtual void SetSize(int Width,int Height)=0;
};

enum MediaDeviceType {
	MEDIA_DEVICE_MCI,
	MEDIA_DEVICE_DIRECTSHOW
};

class MediaPlayer {
	HWND m_hwnd;
	HWND m_hwndParent;
	MediaDeviceType m_MediaDevice;
	Media *m_pMedia;
	LPTSTR m_pszFileName;
	HWND m_hwndPlayBar;
	HIMAGELIST m_himlPlayBar;
	HIMAGELIST m_himlPlayBarHot;
	HIMAGELIST m_himlPlayBarDisabled;
	VolumeSlider *m_pVolumeSlider;
	SeekBar *m_pSeekBar;
	HWND m_hwndCanvas;
	bool m_fShowPlayBar;
	bool m_fShowSeekBar;
	COLORREF m_crBackground;
	COLORREF m_crFullScreenBg;
	HBITMAP m_hbmLogo;
	int m_Zoom;
	int m_XAspect;
	int m_YAspect;
	int m_ForwardSpeed;
	int m_BackwardSpeed;
	bool m_fRepeat;
	bool m_fUserStop;
	bool m_fFullScreen;
	HWND m_hwndFullScreen;
	int m_OldZoom;
	bool m_fFitToWindow;
	bool m_fFullScreenFitToDisplay;
	bool m_fFullScreenHideCursor;
	bool m_fShowCursor;
	bool m_fScreenSaverActive;
	bool m_fSeeking;
	bool m_fMediaChangeInterval;
	MediaMode m_SeekOldMode;
	void CalcDestSize(SIZE *psz);
	HWND CreatePlayBar();
	bool CreateSeekBar();
	bool CreateCanvas();
	bool CreateMedia();
	bool OnCreate(HWND hwnd);
	void OnSize(int Width,int Height);
	void OnHScroll(int Type);
	LRESULT OnNotify(LPNMHDR pnmhdr);
	void OnCommand(int nID,HWND hwndCtrl,UINT uNotifyCode);
	void OnDestroy();
	void OnCanvasPaint();
	void OnCanvasSize(int Width,int Height);
	void OnLButtonDown();
	void OnRButtonDown();
	void OnDropFiles(HDROP hdrop);
	void OnNotifyMode(MediaMode Mode);
	void OnNotifyPos(MediaPosition Pos);
	static LRESULT CALLBACK MediaCanvasWndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam);
	static LRESULT CALLBACK MediaWndProc(HWND hwnd,UINT uMsg,WPARAM wParam,
																LPARAM lParam);
	void SendNotify(int Code);
	bool OnFullScreenCreate(HWND hwnd);
	void OnFullScreenClose();
	void OnFullScreenDestroy();
	static LRESULT CALLBACK FullScreenWndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam);
public:
	static bool Init(HINSTANCE hinst);
	MediaPlayer();
	~MediaPlayer();
	bool Create(HWND hwndParent,int ID,HINSTANCE hinst);
	HWND GetHandle() { return m_hwnd; }
	bool SetMediaDevice(MediaDeviceType Device);
	MediaType GetMediaType() const;
	bool IsOpen() const;
	bool Open(LPCTSTR pszFileName);
	void Close();
	bool Play();
	bool Pause();
	bool Stop();
	bool PrevFrame();
	bool NextFrame();
	bool Backward();
	bool Forward();
	bool SetBackwardSpeed(int Speed);
	bool SetForwardSpeed(int Speed);
	bool Home();
	bool End();
	int GetZoom() const { return m_Zoom; }
	bool SetZoom(int Zoom);
	bool SetAspectRatio(int XAspect,int YAspect);
	int GetXAspect() const { return m_XAspect; }
	int GetYAspect() const { return m_YAspect; }
	void SetRepeat(bool fRepeat);
	bool GetRepeat() const { return m_fRepeat; }
	unsigned int GetVolume() const;
	bool SetVolume(unsigned int Volume);
	bool Mute(bool fMute);
	bool IsMute() const;
	bool GetSourceSize(LPSIZE psz);
	bool GetSourceSize(LPRECT prc);
	bool GetDestSize(LPSIZE psz);
	bool GetDestSize(LPRECT prc);
	MediaPosition GetLength();
	MediaPosition GetStart();
	MediaPosition GetEnd();
	MediaPosition GetPosition();
	MediaMode GetMode() const;
	bool GetFileName(LPTSTR pszFileName,int MaxLength) const;
	LPCTSTR GetFileName() const { return m_pszFileName; }
	bool HasAudio();
	bool FullScreen(bool fFullScreen);
	bool IsFullScreen() const { return m_fFullScreen; }
	void FitToWindow(bool fFit);
	void FullScreenFitToDisplay(bool fFit);
	void FullScreenHideCursor(bool fHide);
	void AdjustRect(LPRECT prc);
	void ShowPlayBar(bool fShow);
	void ShowSeekBar(bool fShow);
	void ShowTipHelp(bool fShow);
	void SetBackgroundColor(COLORREF cr);
	void SetFullScreenBgColor(COLORREF cr);
	void SetTipColor(COLORREF crText,COLORREF crBack);
	void SetVolumeColor(COLORREF crColor);
	void SetSeekBarBarColor(COLORREF crColor);
	bool SetPlayBarImage(HIMAGELIST himl,HIMAGELIST himlHot,
													HIMAGELIST himlDisabled);
	void GetCanvasRect(RECT *prc);
	void SetLogo(HBITMAP hbm);
};


#define MEDIAPLAYER_NOTIFY_ZOOMCHANGE			1	// \{ύXꂽ
#define MEDIAPLAYER_NOTIFY_END					2	// Ō܂ōĐ
#define MEDIAPLAYER_NOTIFY_REPEAT				3	// JԂύXꂽ
#define MEDIAPLAYER_NOTIFY_LBUTTONDOWN			4	// NbNꂽ
#define MEDIAPLAYER_NOTIFY_RBUTTONDOWN			5	// ENbNꂽ
#define MEDIAPLAYER_NOTIFY_PLAYBAR_RBUTTONDOWN	6
#define MEDIAPLAYER_NOTIFY_SEEKBAR_RBUTTONDOWN	7
#define MEDIAPLAYER_NOTIFY_FULLSCREENEND		8	// tXN[I


#endif	/* ndef MEDIA_H */
