/******************************************************************************
*                                                                             *
*    MediaDirectShow.h                      Copyright(c) 2008-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef MEDIA_DIRECTSHOW_H
#define MEDIA_DIRECTSHOW_H


#include <dshow.h>
#include <qedit.h>
#include "Media.h"


class DirectShowMedia : public Media {
	IGraphBuilder *m_pGraphBuilder;
	IMediaControl *m_pMediaControl;
	IMediaEventEx *m_pMediaEvent;
	IMediaSeeking *m_pMediaSeeking;
	IVideoWindow *m_pVideoWindow;
	unsigned int m_TimerID;
	bool SetRate(int Rate);
	bool GetRange(MediaPosition *pStart,MediaPosition *pEnd);
	bool SetMediaVolume(unsigned int Volume);
	static LRESULT CALLBACK FormWndProc(HWND hwnd,UINT uMsg,WPARAM wParam,
																LPARAM lParam);
	bool CheckMediaType(bool *pfHasVideo,bool *pfHasAudio);
	bool HasVideo();
	bool BeginTimer();
	void EndTimer();
	bool Run();
	MediaPosition GetTimePerFrame();
public:
	static bool Init(HINSTANCE hinst);
	DirectShowMedia();
	~DirectShowMedia();
	bool Create(HWND hwndParent,int ID,HINSTANCE hinst);
	void Destroy();
	bool Open(LPCTSTR pszFileName);
	void Close();
	bool Play();
	bool Pause();
	bool Stop();
	bool PrevFrame();
	bool NextFrame();
	bool Backward();
	bool Forward();
	bool SetBackwardSpeed(int Speed);
	bool SetForwardSpeed(int Speed);
	bool Home();
	bool End();
	bool SetVolume(unsigned int Volume);
	bool Mute(bool fMute);
	bool GetSourceSize(LPSIZE psz);
	bool GetDestSize(LPSIZE psz);
	MediaPosition GetLength();
	MediaPosition GetStart();
	MediaPosition GetEnd();
	MediaPosition GetPosition();
	bool SetPosition(MediaPosition Position);
	bool HasAudio();
	void Move(int x,int y,int Width,int Height);
	void SetSize(int Width,int Height);
};


#endif	// ndef MEDIA_DIRECTSHOW_H
