/******************************************************************************
*                                                                             *
*    Playlist.h                             Copyright(c) 2005-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef PLAYLIST_H
#define PLAYLIST_H


#include "FileList.h"
#include "History.h"


class Playlist {
public:
	enum {
		NOTIFY_SELCHANGE=1,	// IύXꂽ
		NOTIFY_PLAY,		// ĐJn
		NOTIFY_FILEDELETE,	// Đ̃t@C폜ꂽ
		NOTIFY_CLOSE		// EBhEꂽ
	};
	enum FileNameType {
		FILENAME_FILENAME,
		FILENAME_FULLPATH,
		FILENAME_DIRECTORY_FILENAME
	};
private:
	HWND m_hwnd;
	HWND m_hwndToolbar;
	HIMAGELIST m_himlToolbar;
	HIMAGELIST m_himlToolbarHot;
	HIMAGELIST m_himlToolbarDisabled;
	HWND m_hwndList;
	WNDPROC m_pfnOldListProc;
	int m_nItemHeight;
	int m_nTextHeight;
	HFONT m_hfont;
	HFONT m_hfontPlaying;
	int m_nHorizontalExtent;
	int m_nPlayingFile;
	int m_DropInsertPos;
	UINT m_DragTimerID;
	BOOL m_fFindSubDirectory;
	TCHAR m_szFileMasks[MAX_FILE_MASK_LENGTH];
	FileNameType m_FileNameType;
	HMENU m_hmenuRecentFiles;
	int m_nMaxFileHistory;
	History m_History;
	COLORREF m_crTextColor;
	COLORREF m_crBgColor;
	COLORREF m_crSelTextColor;
	COLORREF m_crSelBgColor;
	COLORREF m_crPlayingTextColor;
	COLORREF m_crPlayingSelTextColor;
	COLORREF m_crPlayedTextColor;
	HBRUSH m_hbrBackground;
	HWND CreateToolbar();
	void SetToolbarStat();
	BOOL CreateList();
	LPCTSTR GetDisplayFileName(LPCTSTR pszFilePath,LPTSTR pszFileName);
	int CalcListHExtent();
	int CalcFileNameHExtent(LPCTSTR pszFileName);
	void DrawInsertMark(int Pos);
	static LRESULT CALLBACK ListWndProc(HWND hwnd,UINT uMsg,WPARAM wParam,
																LPARAM lParam);
	BOOL OnCreate(HWND hwnd,LPCREATESTRUCT pcs);
	void OnSize(int nWidth,int nHeight);
	void OnDrawItem(LPDRAWITEMSTRUCT pdis);
	LRESULT OnNotify(LPNMHDR pnmhdr);
	void OnDropFiles(HDROP hdrop);
	//void OnFontChange();
	void OnCommand(int nID,HWND hwndCtrl,UINT uNotifyCode);
	void OnClose();
	void OnDestroy();
	static LRESULT CALLBACK PlaylistWndProc(HWND hwnd,UINT uMsg,WPARAM wParam,
																LPARAM lParam);
	void SendNotify(int Type);
	BOOL LoadM3UFile(LPCTSTR pszFileName);
public:
	static bool Init(HINSTANCE hinst);
	Playlist();
	~Playlist();
	HWND GetHandle() { return m_hwnd; }
	bool Create(HWND hwndOwner,HINSTANCE hinst);
	void Show(BOOL fShow);
	int GetCount();
	int GetSel();
	BOOL SetSel(int nIndex);
	BOOL SetPlaying(int nIndex);
	int GetPlaying() { return m_nPlayingFile; }
	BOOL GetFile(int nIndex,LPTSTR pszFileName);
	BOOL AddFile(LPCTSTR pszFileName);
	BOOL AddFiles(const FileList *pFileList);
	BOOL AddDirectoryFiles(LPCTSTR pszDirectory);
	BOOL AddFileOrDirectoryFiles(LPCTSTR pszPath);
	BOOL AddDropFiles(HDROP hdrop);
	BOOL InsertFile(int nIndex,LPCTSTR pszFileName);
	BOOL DeleteFile(int nIndex);
	void Clear();
	int FindFile(LPCTSTR pszFileName);
	void VisibleFile(int Index);
	HICON GetFileIcon(int Index);
	bool GetDisplayFileName(int Index,LPTSTR pszFileName);
	bool IsFilePlayed(int Index);
	BOOL Load(LPCTSTR pszFileName);
	BOOL Save(LPCTSTR pszFileName);
	BOOL LoadList();
	BOOL SaveList();
	BOOL ClearSaveList();
	void ShowTipHelp(BOOL fShow);
	void FindSubDirectory(BOOL fFind);
	void SetFileMasks(LPCTSTR pszMasks);
	void SetFileNameType(FileNameType Type);
	bool LoadHistory(LPTSTR pszFileName);
	bool SaveHistory(LPTSTR pszFileName);
	void ClearHistory();
	void SetMaxHistory(int Max);
	int GetHistoryCount();
	void SetColors(COLORREF crTextColor,COLORREF crBgColor,
				COLORREF crSelTextColor,COLORREF crSelBgColor,
				COLORREF crPlayingTextColor,COLORREF crPlayingSelTextColor,
				COLORREF crPlayedTextColor);
	void SetTipColor(COLORREF crText,COLORREF crBack);
	bool SetToolbarImage(HIMAGELIST himl,HIMAGELIST himlHot,
													HIMAGELIST himlDisabled);
};


#endif	/* ndef PLAYLIST_H */
