/******************************************************************************
*                                                                             *
*    Settings.cpp                           Copyright(c) 2005-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <tchar.h>
#include "Settings.h"




/*
	𕄍tɕϊ
*/
static int StrToInt(LPCTSTR pszValue)
{
	int nValue;
	LPCTSTR p;

	nValue=0;
	p=pszValue;
	if (*p=='-' || *p=='+')
		p++;
	while (*p>='0' && *p<='9') {
		nValue=nValue*10+(*p-'0');
		p++;
	}
	return pszValue[0]=='-'?-nValue:nValue;
}


/*
	𕄍Ȃɕϊ
*/
static unsigned int StrToUInt(LPCTSTR pszValue)
{
	unsigned int uValue;
	LPCTSTR p;

	uValue=0;
	p=pszValue;
	while (*p>='0' && *p<='9') {
		uValue=uValue*10+(*p-'0');
		p++;
	}
	return uValue;
}


/*
	16i̕𐔒lɕϊ
*/
static int HexToNum(TCHAR cCode)
{
	if (cCode>='0' && cCode<='9')
		return cCode-'0';
	if (cCode>='A' && cCode<='F')
		return cCode-'A'+10;
	if (cCode>='a' && cCode<='f')
		return cCode-'a'+10;
	return 0;
}


Settings::Settings()
{
	m_OpenFlags=0;
}


Settings::~Settings()
{
	if ((m_OpenFlags&OPEN_WRITE)!=0)
		WritePrivateProfileString(NULL,NULL,NULL,m_szFileName);	// Flush
}


bool Settings::Open(LPCTSTR pszFileName,LPCTSTR pszSection,unsigned int Flags)
{
	lstrcpy(m_szFileName,pszFileName);
	lstrcpy(m_szSection,pszSection);
	m_OpenFlags=Flags;
	return true;
}


void Settings::Close()
{
	m_OpenFlags=0;
}


bool Settings::Clear()
{
	return WritePrivateProfileSection(m_szSection,TEXT("\0"),m_szFileName)!=FALSE;
}


bool Settings::ReadInt(LPCTSTR pszValueName,int *pData)
{
	TCHAR szValue[16];

	if (!ReadString(pszValueName,szValue,16) || szValue[0]=='\0')
		return false;
	*pData=StrToInt(szValue);
	return true;
}


bool Settings::WriteInt(LPCTSTR pszValueName,int Data)
{
	TCHAR szValue[16];

	wsprintf(szValue,TEXT("%d"),Data);
	return WriteString(pszValueName,szValue);
}


bool Settings::ReadUInt(LPCTSTR pszValueName,unsigned int *pData)
{
	TCHAR szValue[16];

	if (!ReadString(pszValueName,szValue,16) || szValue[0]=='\0')
		return false;
	*pData=StrToUInt(szValue);
	return true;
}


bool Settings::WriteUInt(LPCTSTR pszValueName,unsigned int Data)
{
	TCHAR szValue[16];

	wsprintf(szValue,TEXT("%u"),Data);
	return WriteString(pszValueName,szValue);
}


bool Settings::ReadIntArray(LPCTSTR pszValueName,int *pData,int *pLength)
{
	LPTSTR pszData,p,q;
	TCHAR szTmp[12];
	int i;

	pszData=new TCHAR[*pLength*12];
	if (!ReadString(pszValueName,pszData,*pLength*12) || pszData[0]=='\0') {
		delete [] pszData;
		return false;
	}
	p=pszData;
	for (i=0;i<*pLength && *p!='\0';i++) {
		q=szTmp;
		while (true) {
			if (*p==',' || *p==' ' || *p=='\t' || *p=='\0'
								|| q-szTmp==sizeof(szTmp)/sizeof(TCHAR)-1) {
				*q='\0';
				pData[i]=StrToInt(szTmp);
				while (*p!=',' && *p!=' ' && *p!='\t'&& *p!='\0')
					p++;
				if (*p!='\0')
					p++;
				break;
			}
			*q++=*p++;
		}
	}
	if (i<*pLength)
		*pLength=i;
	delete [] pszData;
	return true;
}


bool Settings::WriteIntArray(LPCTSTR pszValueName,const int *pData,int Length)
{
	bool fRet;
	LPTSTR pszData,p;
	int i;

	pszData=new TCHAR[Length*12];
	p=pszData;
	for (i=0;i<Length;i++) {
		p+=wsprintf(p,TEXT("%d"),pData[i]);
		*p++=',';
	}
	*(p-1)='\0';
	fRet=WriteString(pszValueName,pszData);
	delete [] pszData;
	return fRet;
}


bool Settings::ReadString(LPCTSTR pszValueName,LPTSTR pszData,unsigned int Max)
{
	TCHAR cBack[2];

	cBack[0]=pszData[0];
	if (Max>1)
		cBack[1]=pszData[1];
	GetPrivateProfileString(m_szSection,pszValueName,TEXT("\x1A"),pszData,Max,
																m_szFileName);
	if (pszData[0]=='\x1A') {
		pszData[0]=cBack[0];
		if (Max>1)
			pszData[1]=cBack[1];
		return false;
	}
	return true;
}


bool Settings::WriteString(LPCTSTR pszValueName,LPCTSTR pszData)
{
	return WritePrivateProfileString(m_szSection,pszValueName,pszData,
														m_szFileName)!=FALSE;
}


bool Settings::ReadBoolean(LPCTSTR pszValueName,bool *pfData)
{
	TCHAR szData[5];

	if (!ReadString(pszValueName,szData,5))
		return false;
	if (lstrcmpi(szData,TEXT("yes"))==0)
		*pfData=true;
	else if (lstrcmpi(szData,TEXT("no"))==0)
		*pfData=false;
	else
		return false;
	return true;
}


bool Settings::ReadBoolean(LPCTSTR pszValueName,BOOL *pfData)
{
	bool f;

	if (!ReadBoolean(pszValueName,&f))
		return false;
	*pfData=f;
	return true;
}


bool Settings::WriteBoolean(LPCTSTR pszValueName,bool fData)
{
	return WriteString(pszValueName,fData?TEXT("yes"):TEXT("no"));
}


bool Settings::ReadColor(LPCTSTR pszValueName,COLORREF *pcrData)
{
	TCHAR szText[8];

	if (!ReadString(pszValueName,szText,8)
									|| szText[0]!='#' || lstrlen(szText)!=7)
		return false;
	*pcrData=RGB((HexToNum(szText[1])<<4) | HexToNum(szText[2]),
				(HexToNum(szText[3])<<4) | HexToNum(szText[4]),
				(HexToNum(szText[5])<<4) | HexToNum(szText[6]));
	return true;
}


bool Settings::WriteColor(LPCTSTR pszValueName,COLORREF crData)
{
	TCHAR szText[8];

	wsprintf(szText,TEXT("#%02X%02X%02X"),
						GetRValue(crData),GetGValue(crData),GetBValue(crData));
	return WriteString(pszValueName,szText);
}
