/******************************************************************************
*                                                                             *
*    Theme.cpp                              Copyright(c) 2007-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <tchar.h>
#include <commctrl.h>
#include "LafPlayer.h"
#include "Theme.h"
#include "FileList.h"
#include "Util.h"




ThemePlugin::ThemePlugin()
{
	m_hLib=NULL;
}


ThemePlugin::~ThemePlugin()
{
	if (m_hLib!=NULL)
		FreeLibrary(m_hLib);
}


bool ThemePlugin::Load(LPCTSTR pFileName)
{
	HINSTANCE hLib;
	LPThemeGetVersionFunc pfnGetVersion;
	LPThemeGetInfoFunc pfnGetInfo;

	hLib=static_cast<HINSTANCE>(LoadLibrary(pFileName));
	if (hLib==NULL)
		return false;
	pfnGetVersion=(LPThemeGetVersionFunc)
								GetProcAddress(hLib,"LPThemeGetVersion");
	pfnGetInfo=(LPThemeGetInfoFunc)GetProcAddress(hLib,"LPThemeGetInfo");
	if (pfnGetVersion==NULL || pfnGetInfo==NULL
								|| (*pfnGetVersion)()!=LP_THEME_VERSION) {
		FreeLibrary(hLib);
		return false;
	}
	m_hLib=hLib;
	lstrcpy(m_FileName,pFileName);
	(*pfnGetInfo)(&m_Info);
	return true;
}


void ThemePlugin::GetInfo(ThemePluginInfo *pInfo) const
{
	pInfo->Flags=m_Info.Flags;
#ifndef UNICODE
	WideCharToMultiByte(CP_ACP,0,m_Info.pszName,-1,
									pInfo->Name,LP_THEME_MAX_NAME,NULL,NULL);
	WideCharToMultiByte(CP_ACP,0,m_Info.pszCopyright,-1,
							pInfo->Copyright,LP_THEME_MAX_COPYRIGHT,NULL,NULL);
	WideCharToMultiByte(CP_ACP,0,m_Info.pszDescription,-1,
						pInfo->Description,LP_THEME_MAX_DESCRIPTION,NULL,NULL);
#else
	lstrcpy(pInfo->Name,m_Info.pszName);
	lstrcpy(pInfo->Copyright,m_Info.pszCopyright);
	lstrcpy(pInfo->Description,m_Info.pszDescription);
#endif
}


bool ThemePlugin::GetToolbarInfo(LPThemeToolbarInfo *pInfo) const
{
	LPThemeGetToolbarInfoFunc pfnGetToolbarInfo;

	pfnGetToolbarInfo=(LPThemeGetToolbarInfoFunc)
								GetProcAddress(m_hLib,"LPThemeGetToolbarInfo");
	if (pfnGetToolbarInfo==NULL)
		return false;
	return (*pfnGetToolbarInfo)(pInfo)!=FALSE;
}


HBITMAP ThemePlugin::LoadBitmap(int Type) const
{
	UINT ID;

	switch (Type) {
	case LP_IMAGE_TYPE_LOGO:
		return ::LoadBitmap(m_hLib,MAKEINTRESOURCE(LP_THEME_IDB_LOGO));
	case LP_IMAGE_TYPE_TOOLBAR:
		ID=LP_THEME_IDB_TOOLBAR;
		break;
	case LP_IMAGE_TYPE_PLAYBAR:
		ID=LP_THEME_IDB_PLAYBAR;
		break;
	case LP_IMAGE_TYPE_PLAYLIST:
		ID=LP_THEME_IDB_PLAYLIST;
		break;
	}
	return (HBITMAP)LoadImage(m_hLib,MAKEINTRESOURCE(ID),IMAGE_BITMAP,0,0,
															LR_DEFAULTSIZE);
}


HIMAGELIST ThemePlugin::LoadImageList(int Type,ButtonStat Stat) const
{
	UINT ID;
	LPThemeToolbarInfo ToolbarInfo;

	switch (Type) {
	case LP_TOOLBAR_TYPE_TOOLBAR:
		ID=LP_THEME_IDB_TOOLBAR+Stat;
		break;
	case LP_TOOLBAR_TYPE_PLAYBAR:
		ID=LP_THEME_IDB_PLAYBAR+Stat;
		break;
	case LP_TOOLBAR_TYPE_PLAYLIST:
		ID=LP_THEME_IDB_PLAYLIST+Stat;
		break;
	}
	ToolbarInfo.Type=Type;
	GetToolbarInfo(&ToolbarInfo);
	return ImageList_LoadImage(m_hLib,MAKEINTRESOURCE(ID),ToolbarInfo.Width,1,
				ToolbarInfo.TransparentColor,IMAGE_BITMAP,LR_CREATEDIBSECTION |
				((ToolbarInfo.Flags&LP_TOOLBAR_FLAG_MAP3DCOLORS)!=0?
														LR_LOADMAP3DCOLORS:0));
}




ThemePluginList::ThemePluginList()
{
	m_NumPlugins=0;
	m_pList=NULL;
}


ThemePluginList::~ThemePluginList()
{
	if (m_pList!=NULL)
		delete [] m_pList;
}


bool ThemePluginList::LoadPlugins(LPCTSTR pDirectory)
{
	FileList Files;
	int i;

	if (!Files.AddDirectoryFiles(pDirectory,TEXT("*.lpres"),FALSE))
		return false;
	if (Files.GetCount()==0)
		return true;
	m_pList=new ThemePlugin[Files.GetCount()];
	for (i=0;i<Files.GetCount();i++) {
		TCHAR szFileName[MAX_PATH];

		Files.GetFile(i,szFileName);
		if (m_pList[m_NumPlugins].Load(szFileName))
			m_NumPlugins++;
	}
	return true;
}
